/*
 * Updated 2007/01/27
 * Ver.1.01
 * charset = UTF-8
 * 
 * Mail : info@graviness.com
 * Home : http://www.graviness.com/
 */



//import graviness-sys-Common.js
//import graviness-ajax-Ajaxor.js

/**
 * Ajaxorクラスを継承したシンプルなAjaxクラス．
 * 
 * HTTPメソッドをPOSTに限定したシンプルなAjaxクラスです．
 * 記述が簡略化できます．
 * 
 * @see graviness-ajax-Ajax
 */
function class__SimpleAjaxor__extends__Ajaxor__(window) {
	var className = Object.getClassName(arguments.callee);

	var $pairs = Object.privateId();
	var $completedListeners = Object.privateId();

	/**
	 * ペアクラス．
	 * 
	 * keyとvalueを管理する．
	 * @param sKey	キー文字列．
	 * @param sValue	値文字列．
	 */
	function Pair(sKey, sValue) {
		this.sKey = sKey;
		this.sValue = sValue;
	}

	/**
	 * SimpleAjaxorクラスのインスタンスを構築します．
	 */
	var F = window[className] = function() {
		this.__super__(); // super();

		// 送信パラメータを示すkeyとvalueの集合．
		this[$pairs] = new Array();
		// 受信完了時リスナを要素に持つ配列．
		this[$completedListeners] = new Array();

		this.addReadyStateChangeListener(function(that, ls) {
			return function(readyState, superKernel) {
				if (readyState != F.STATE_COMPLETED)
					return;

				for (var i = 0, n = ls.length; i < n; i++) {
					var l = ls[i];
					if (l === null)
						continue;

					l(superKernel.getResponseXML(), that);
				}
			}
		}(this, this[$completedListeners]));
	};

	/**
	 * @see graviness-ajax-Ajaxor
	 */
	F.STATE_UNINITIALIZED = Ajaxor.STATE_UNINITIALIZED;
	F.STATE_LOADING = Ajaxor.STATE_LOADING;
	F.STATE_LOADED = Ajaxor.STATE_LOADED;
	F.STATE_INTERACTIVE = Ajaxor.STATE_INTERACTIVE;
	F.STATE_COMPLETED = Ajaxor.STATE_COMPLETED;

	var FP = Object.getPrototype(arguments.callee, window);

	/**
	 * 送信パラメータを追加します．
	 * 
	 * 指定するキーや値はエンコード前の状態にしておきます．
	 * @param key	キー．nullは指定できません．
	 * @param value	キーに対応する値．nullは指定できません．
	 * @return 指定されたキーの前の値．それがないとき，null．
	 * @throws NullPointerException
	 * 	keyまたはvalueが，nullまたはundefinedのとき．
	 */
	FP.addParameter = function(key, value) {
		if (key === null || key === void 0
			|| value === null || value === void 0)
			throw "NullPointerException@"
				+ className + "#addParamter(key, value)";

		var sKey = key.toString(), sValue = value.toString();

		for (var i = 0, n = this[$pairs].length; i < n; i++) {
			var pair = this[$pairs][i];
			if (pair === null)
				continue;

			if (pair.sKey === sKey) {
				var oldValue = pair.sValue;
				pair.sValue = sValue;
				return oldValue;
			}
		}

		this[$pairs].push(new Pair(sKey, sValue));

		return null;
	};

	/**
	 * 送信パラメータのうち，指定したキーを持つパラメータを削除します．
	 * 
	 * @param key	削除するキー．
	 * @return パラメータでそのキーに対応する値．
	 * 	対応する値がない場合，null．
	 * @throws NullPointerException	keyが，nullまたはundefinedのとき．
	 */
	FP.removeParameter = function(key) {
		if (key === null || key === void 0)
			throw "NullPointerException@"
				+ className + "#removeParamter(key)";

		var sKey = key.toString();

		for (var i = 0, n = this[$pairs].length; i < n; i++) {
			var pair = this[$pairs][i];
			if (pair === null)
				continue;

			if (pair.sKey === sKey) {
				this[$pairs][i] = null;
				return pair.sValue;
			}
		}

		return null;
	};

	/**
	 * 全ての送信パラメータを文字列として取得します．
	 * 
	 * 実際にWebサーバに送信する形式で取得します．
	 * @return 送信パラメータ．
	 */
	FP.getAllParamterString = function() {
		var rs = new Array();

		for (var i = 0, n = this[$pairs].length; i < n; i++) {
			var pair = this[$pairs][i];
			if (pair === null)
				continue;

			rs.push(encodeURI(pair.sKey) + "=" + encodeURI(pair.sValue));
		}

		return rs.join("&");
	};

	// つぶやき：super.openって書きたい・・・
	/**
	 * 要求を出すサーバを設定し，初期化します．
	 * 
	 * @param uri	送信先URI．
	 * 	セキュリティ上，異なるドメインを指定できません．
	 * @param async	真のとき非同期通信，偽のとき同期通信．
	 * 	真のとき，JavaScript関数の動作は，サーバから応答が返らなくても
	 * 	続けることができます．指定しないとき(デフォルト)は，非同期通信です．
	 * @throws uriに当該URIと異なるドメインを持つURIを
	 * 	指定したとき(セキュリティの関係)．
	 */
	FP.openConnection = function(uri, async) {
		if (async === void 0)
			async = true;

		this.open("POST", uri, async);

		// POST時の必須処理．
		this.setRequestHeader(
			"Content-Type", "application/x-www-form-urlencoded");
	};

	// つぶやき：super.sendって書きたい・・・
	/**
	 * Webサーバに，addParameterで設定したデータを送信します．
	 */
	FP.sendParameter = function() {
		this.send(this.getAllParamterString());
	};

	/**
	 * 受信完了時のリスナ(関数)を登録します．
	 * 
	 * 登録されたリスナは，受信完了時(readyState == 4)に一度だけ実行されます．
	 * リスナとなる関数は次の形式を持ちます．
	 * 	function listener(XMLObject responseXML, SimpleAjaxor simpleAjaxor)
	 * responseXMLは，取得したXMLオブジェクト，
	 * simpleAjaxorは，このインスタンスです．
	 * 
	 * @param listener	リスナとなる関数．関数は次の形式を持ちます．
	 * 	function(XMLObject responseXML, SimpleAjaxor simpleAjaxor)
	 * @throws IllegalArgumentsException	listenerが不正なとき．
	 */
	FP.addCompletedListener = function(listener) {
		if (!Boolean(listener) || typeof listener != "function")
			throw "IllegalArgumentsException@" + className
				+ "#addCompletedListener(Function)";

		this[$completedListeners].push(listener);
	};

	/**
	 * 登録した状態変更リスナ(関数)を削除します．
	 * 
	 * @param listener	削除するリスナ(関数)．
	 * @return	削除に成功したとき真．listenerが存在しないなど，失敗したとき偽．
	 * @throws IllegalArgumentsException	listenerが不正なとき．
	 */
	FP.removeCompletedListener = function(listener) {
		if (!Boolean(listener) || typeof listener != "function")
			throw "IllegalArgumentsException@" + className +
				"#removeCompletedListener(Function)";

		var ls = this[$completedListeners];
		for (var i = 0, n = ls.length; i < n; i++) {
			if (ls[i] === listener) {
				ls[i] = null;
				return true;
			}
		}

		return false;
	};

	/**
	 * 受信完了後の，受信データの有効/無効を調べます．
	 * 
	 * 実際には，受信完了(readyState==4)かつHTTPステータスコードが
	 * 200(OK)のとき真を返します．それ以外は偽です．
	 * @return 受信データが利用できるとき，真．
	 * 	なんらかの理由により受信データが利用できない状態のとき，偽．
	 */
	FP.isAvailable = function() {
		return this.getReadyState() == F.STATE_COMPLETED
			&& this.getStatus() == 200;
	};

} class__SimpleAjaxor__extends__Ajaxor__(window);
